#!/usr/bin/python3

import unittest
import tempfile
import os
import shutil
import sys

sys.path.insert(0, "..")
from apt_clone import AptClone

class TestMergeSources(unittest.TestCase):
    def test_merge_sources(self):
        clone = AptClone()
        tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmpdir)
        sources_list = os.path.join(tmpdir, "etc", "apt", "sources.list")
        os.makedirs(os.path.dirname(sources_list))
        shutil.copy('data/lucid-sources.list', sources_list)
        backup = os.path.join(tmpdir, "etc", "apt", "sources.list.apt-clone")
        shutil.copy('data/natty-sources.list', backup)
        clone._rewrite_sources_list(tmpdir, 'natty')
        with open(sources_list) as fp:
            # Tally the occurances of every source line.
            from collections import defaultdict
            tally = defaultdict(int)
            for line in fp:
                if line != '\n' and not line.startswith('#'):
                    tally[line] += 1
            # There should not be any duplicate source lines.
            for line, count in tally.items():
                self.assertTrue(count == 1, '"%s" occurred %d times.'
                                % (line, count))

            # Check for extras, others...
            l = (('partner',
                  'deb http://archive.canonical.com/ubuntu natty partner\n'),
                 ('extras',
                  'deb http://extras.ubuntu.com/ubuntu natty main\n'),
                 ('main',
                  'deb http://gb.archive.ubuntu.com/ubuntu/ natty main restricted\n'))
            for pocket, match in l:
                fp.seek(0)
                found = False
                for line in fp:
                    if line == match:
                        found = True
                self.assertTrue(found,
                        '%s repository not present or disabled.' % pocket)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

import logging
import os
import shutil
import sys
import tarfile
import tempfile
import unittest
import io

import apt
import apt_pkg
import distro_info

sys.path.insert(0, "..")
from apt_clone import AptClone

#apt.apt_pkg.config.set("Debug::pkgProblemResolver", "1")

class TestCloneUpgrade(unittest.TestCase):

    @unittest.skip("need to update apt-clone-state-ubuntu.tar.gz first")
    def test_clone_upgrade_regression(self):
        """ regression test against known installs """
        new = self._create_fake_upgradable_root("natty", meta="ubuntu-desktop")
        self.addCleanup(shutil.rmtree, new)
        cache = apt.Cache(rootdir=new)
        clone = AptClone()
        clone._restore_package_selection_in_cache(
            "./data/regression/apt-clone-state-ubuntu.tar.gz", cache)
        self.assertTrue(len(cache.get_changes()) > 0)

    def test_clone_upgrade_synthetic(self):
        """ test clone upgrade with on-the-fly generated chroots """
        supported = distro_info.UbuntuDistroInfo().supported()
        for meta in ["ubuntu-standard", "ubuntu-desktop", "kubuntu-desktop",
                     "xubuntu-desktop"]:
            arch = apt_pkg.config.find("APT::Architecture")
            if arch in ['s390x'] and meta in ['ubuntu-desktop', 'kubuntu-desktop']:
                continue
            logging.info("testing %s" % meta)
            old = self._create_fake_upgradable_root(supported[-2], meta=meta)
            self.addCleanup(shutil.rmtree, old)
            # create statefile based on the old data
            with tarfile.open("lala.tar.gz", "w:gz") as state:
                state.add(
                    os.path.join(old, "var", "lib", "apt-clone",
                                 "installed.pkgs"),
                    arcname = "./var/lib/apt-clone/installed.pkgs")
            # create new fake environment and try to upgrade
            new = self._create_fake_upgradable_root(supported[-1], meta=meta)
            self.addCleanup(shutil.rmtree, new)
            cache = apt.Cache(rootdir=new)
            clone = AptClone()
            clone._restore_package_selection_in_cache("lala.tar.gz", cache, protect_installed=True)
            self.assertFalse(cache[meta].marked_delete,
                             "package %s marked for removal" % meta)
            self.assertTrue(len(cache.get_changes()) > 0)

    def _ensure_arch_available_on_server(self, server, from_dist, arch):
        # py2/py3 compat
        try:
            from urllib.request import urlopen
        except ImportError:
            from urllib import urlopen
            urlopen  # pyflakes
        uri = "http://%s/dists/%s/main/binary-%s/" % (server, from_dist, arch)
        try:
            fp = urlopen(uri)
            fp.close()
        except IOError:
            return self.skipTest("can not find %s" % uri)
        if fp.getcode() == 404:
            return self.skipTest("can not find %s" % uri)

    def _create_fake_upgradable_root(self, from_dist,
                                     meta="ubuntu-desktop",
                                     tmpdir=None):
        if tmpdir is None:
            tmpdir = tempfile.mkdtemp()
        sources_list = os.path.join(tmpdir, "etc", "apt", "sources.list")
        if not os.path.exists(os.path.dirname(sources_list)):
            os.makedirs(os.path.dirname(sources_list))
        keyring_src = os.path.join("/usr", "share", "keyrings", "ubuntu-archive-keyring.gpg")
        trusted_d = os.path.join(tmpdir, "etc", "apt", "trusted.gpg.d")
        if not os.path.exists(trusted_d):
            os.makedirs(trusted_d)
        shutil.copy(keyring_src, trusted_d)
        arch = apt_pkg.config.find("APT::Architecture")
        if arch in ['i386', 'amd64']:
            server = 'archive.ubuntu.com/ubuntu'
        else:
            server = 'ports.ubuntu.com/ubuntu-ports'
        # check that the server actually has the given arch, this may not
        # be the case if a architecture is new (like ppc64el)
        self._ensure_arch_available_on_server(server, from_dist, arch)
        with open(os.path.join(sources_list), "w") as fp:
            fp.write("""
deb http://%s %s main restricted universe multiverse
""" % (server, from_dist))
        cache = apt.Cache(rootdir=tmpdir)
        cache.update()
        cache.open()
        if not cache[meta].is_installed:
            cache[meta].mark_install()
            installed_pkgs = os.path.join(tmpdir, "var", "lib", "apt-clone", "installed.pkgs")
            if not os.path.exists(os.path.dirname(installed_pkgs)):
                os.makedirs(os.path.dirname(installed_pkgs))
            dpkg_status = os.path.join(tmpdir, "var", "lib", "dpkg", "status")
            if not os.path.exists(os.path.dirname(dpkg_status)):
                os.makedirs(os.path.dirname(dpkg_status))
            with io.open(dpkg_status, "w", encoding="utf-8") as dpkg:
                with open(installed_pkgs, "w") as installed:
                    for pkg in cache:
                        if pkg.marked_install:
                            s = str(pkg.candidate.record)
                            s = s.replace("Package: %s\n" % pkg.name,
                                          "Package: %s\n%s\n" % (
                                    pkg.name, "Status: install ok installed"))
                            if sys.version < '3':
                                s = unicode(s, encoding='utf-8')
                            dpkg.write(u"%s\n" % s)
                            installed.write("%s %s %s\n" % (pkg.name,
                                                            pkg.candidate.version,
                                                            int(pkg.is_auto_installed)))
        return tmpdir


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function

import os
import sys
import unittest

# this is important
os.environ["LANG"] = "C"


sys.path.insert(0, os.path.join(os.path.dirname(__file__), ".."))
from apt_clone import AptClone


class MockTar(object):
    def add(self, source, arcname):
        with open(source, "rb") as f:
            self.data = f.read().decode("utf-8")


class TestClone(unittest.TestCase):

    def setUp(self):
        self.apt_clone = AptClone()
        self.test_sources_fname = "test-sources.list"
        with open(self.test_sources_fname, "wb") as f:
            f.write(u"""# äüö
deb http://mvo:secret@archive.u.c/ ubuntu main
""".encode("utf-8"))

    def tearDown(self):
        os.unlink(self.test_sources_fname)

    def test_scrub_file_from_passwords(self):
        """Regression test for utf8 crash LP: #1309447"""
        mock_tar = MockTar()
        self.apt_clone._add_file_to_tar_with_password_check(
            mock_tar, self.test_sources_fname, scrub=True, 
            arcname="some-archname")
        # see if we got the expected data
        self.assertNotIn("mvo:secret", mock_tar.data)
        self.assertEqual(mock_tar.data, u"""# äüö
deb http://USERNAME:PASSWORD@archive.u.c/ ubuntu main
""")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

from __future__ import print_function

import apt
import logging
import os
import subprocess
import sys
import unittest

# use the right dir
testdir = os.path.dirname(__file__)
if testdir:
    os.chdir(testdir)
# insert path
sys.path.insert(0, "..")
from apt_clone import AptClone

class MockAptCache(apt.Cache):
    def commit(self, fetchp, installp):
        pass
    def update(self, fetchp):
        pass

class TestClone(unittest.TestCase):

    def test_real(self):
        if os.getuid() != 0:
            print("Skipping because uid != 0")
            return
        # do it
        target = "./test-chroot"
        if not os.path.exists(target):
            os.mkdir(target)
            subprocess.call(["debootstrap", "--arch=i386",
                             "maverick", target])
        # force i386
        with open(os.path.join(target, "etc/apt/apt.conf"), "w") as fp:
            fp.write('APT::Architecture "i386";')

        # restore
        clone = AptClone()
        clone.restore_state(
            "./data/apt-state_chroot_with_vim.tar.gz", target, "maverick")

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()
